@@ -31,6 +31,8 @@
 #include "gnutls_cipher_int.h"
 #include "gnutls_priority.h"
 #include "gnutls_algorithms.h"
+#include "gnutls_db.h"
+#include "gnutls_auth_int.h"
 #ifdef HAVE_ERRNO_H
 #include <errno.h>
 #endif
@@ -46,16 +48,12 @@ void gnutls_free(void *ptr) {
 
 GNUTLS_Version gnutls_get_current_version(GNUTLS_STATE state) {
 GNUTLS_Version ver;
-	ver.local = state->connection_state.version.local;
-	ver.major = state->connection_state.version.major;
-	ver.minor = state->connection_state.version.minor;
+	ver = state->connection_state.version;
 	return ver;
 }
 
 void gnutls_set_current_version(GNUTLS_STATE state, GNUTLS_Version version) {
-	state->connection_state.version.local = version.local;
-	state->connection_state.version.major = version.major;
-	state->connection_state.version.minor = version.minor;
+	state->connection_state.version = version;
 }
 
 int gnutls_is_secure_memory(const void* mem) {
@@ -94,6 +92,8 @@ int gnutls_init(GNUTLS_STATE * state, ConnectionEnd con_end)
 	(*state)->cipher_specs.server_write_key = NULL;
 	(*state)->cipher_specs.client_write_key = NULL;
 
+	(*state)->gnutls_internals.v2_hello = 0;
+	
 	(*state)->gnutls_internals.buffer = NULL;
 	/* SSL3 stuff */
 	(*state)->gnutls_internals.hash_buffer = NULL;
@@ -103,12 +103,24 @@ int gnutls_init(GNUTLS_STATE * state, ConnectionEnd con_end)
 
 	gnutls_set_current_version ( (*state), GNUTLS_TLS1); /* default */
 
-	(*state)->gnutls_internals.KEY = NULL;
-	(*state)->gnutls_internals.client_Y = NULL;
-	(*state)->gnutls_internals.client_p = NULL;
-	(*state)->gnutls_internals.client_g = NULL;
-	(*state)->gnutls_internals.dh_secret = NULL;
+	(*state)->gnutls_key = gnutls_malloc(sizeof(GNUTLS_KEY_A));
+
+	(*state)->gnutls_key->username = NULL; /* no default username */
+	(*state)->gnutls_key->cred = NULL; /* no credentials by default */
 	
+	(*state)->gnutls_key->KEY = NULL;
+	(*state)->gnutls_key->client_Y = NULL;
+	(*state)->gnutls_key->client_p = NULL;
+	(*state)->gnutls_key->client_g = NULL;
+	(*state)->gnutls_key->dh_secret = NULL;
+
+	(*state)->gnutls_key->A = NULL;
+	(*state)->gnutls_key->a = NULL;	
+	(*state)->gnutls_key->x = NULL;
+	(*state)->gnutls_key->u = NULL;
+	(*state)->gnutls_key->B = NULL;
+	(*state)->gnutls_key->b = NULL;	
+
 	(*state)->gnutls_internals.certificate_requested = 0;
 	(*state)->gnutls_internals.certificate_verify_needed = 0;
 
@@ -127,7 +139,7 @@ int gnutls_init(GNUTLS_STATE * state, ConnectionEnd con_end)
 	/* Set default priorities */
 	gnutls_set_cipher_priority( (*state), 2, GNUTLS_RIJNDAEL, GNUTLS_3DES);
 	gnutls_set_compression_priority( (*state), 1, GNUTLS_NULL_COMPRESSION);
-	gnutls_set_kx_priority( (*state), 2, GNUTLS_KX_DHE_DSS, GNUTLS_KX_DHE_RSA);
+	gnutls_set_kx_priority( (*state), 1, GNUTLS_KX_ANON_DH);
 	gnutls_set_mac_priority( (*state), 2, GNUTLS_MAC_SHA, GNUTLS_MAC_MD5);
 
 	(*state)->security_parameters.session_id_size = 0;
@@ -149,6 +161,10 @@ int gnutls_init(GNUTLS_STATE * state, ConnectionEnd con_end)
 /* This function clears all buffers associated with the state. */
 int gnutls_deinit(GNUTLS_STATE * state)
 {
+	/* if the session has failed abnormally it has to be removed from the db */
+	if ( (*state)->gnutls_internals.resumable==RESUME_FALSE) {
+		_gnutls_db_remove_session( (*state), (*state)->security_parameters.session_id, (*state)->security_parameters.session_id_size);
+	}
 	gnutls_free((*state)->connection_state.read_compression_state);
 	gnutls_free((*state)->connection_state.read_mac_secret);
 	gnutls_free((*state)->connection_state.write_compression_state);
@@ -157,6 +173,8 @@ int gnutls_deinit(GNUTLS_STATE * state)
 	gnutls_free((*state)->gnutls_internals.buffer);
 	gnutls_free((*state)->gnutls_internals.buffer_handshake);
 
+	gnutls_clear_creds( *state);
+
 	if ((*state)->connection_state.read_cipher_state != NULL)
 		gnutls_cipher_deinit((*state)->connection_state.read_cipher_state);
 	if ((*state)->connection_state.write_cipher_state != NULL)
@@ -169,11 +187,22 @@ int gnutls_deinit(GNUTLS_STATE * state)
 	secure_free((*state)->cipher_specs.server_write_key);
 	secure_free((*state)->cipher_specs.client_write_key);
 
-	mpi_release((*state)->gnutls_internals.KEY);
-	mpi_release((*state)->gnutls_internals.client_Y);
-	mpi_release((*state)->gnutls_internals.client_p);
-	mpi_release((*state)->gnutls_internals.client_g);
-	mpi_release((*state)->gnutls_internals.dh_secret);
+	mpi_release((*state)->gnutls_key->KEY);
+	mpi_release((*state)->gnutls_key->client_Y);
+	mpi_release((*state)->gnutls_key->client_p);
+	mpi_release((*state)->gnutls_key->client_g);
+
+	mpi_release((*state)->gnutls_key->u);
+	mpi_release((*state)->gnutls_key->a);
+	mpi_release((*state)->gnutls_key->x);
+	mpi_release((*state)->gnutls_key->A);
+	mpi_release((*state)->gnutls_key->B);
+	mpi_release((*state)->gnutls_key->b);
+
+	mpi_release((*state)->gnutls_key->dh_secret);
+	gnutls_free((*state)->gnutls_key);
+
+	gnutls_free((*state)->gnutls_key->username);
 
 	/* free priorities */
 	if ((*state)->gnutls_internals.MACAlgorithmPriority.algorithm_priority!=NULL)
@@ -403,7 +432,6 @@ int gnutls_close_nowait(int cd, GNUTLS_STATE state)
  * send (if called by the user the Content is specific)
  * It is intended to transfer data, under the current state.    
  */
-#define MAX_ENC_LEN 16384
 ssize_t gnutls_send_int(int cd, GNUTLS_STATE state, ContentType type, void *_data, size_t sizeofdata, int flags)
 {
 	uint8 *cipher;
@@ -430,24 +458,25 @@ ssize_t gnutls_send_int(int cd, GNUTLS_STATE state, ContentType type, void *_dat
 	}
 
 	headers[0]=type;
-	headers[1]=state->connection_state.version.major;
-	headers[2]=state->connection_state.version.minor;
+	headers[1]=_gnutls_version_get_major(state->connection_state.version);
+	headers[2]=_gnutls_version_get_minor(state->connection_state.version);
 	
 	for (i = 0; i < iterations; i++) {
 		cipher_size = _gnutls_encrypt( state, &data[i*Size], Size, &cipher, type);
-		if (cipher_size<=0) return cipher_size;
+		if (cipher_size <= 0) return cipher_size; /* error */
+		
 #ifdef WORDS_BIGENDIAN
 		length = cipher_size;
 #else
 		length = byteswap16(cipher_size);
 #endif
 		memmove( &headers[3], &length, sizeof(uint16));
-		if (_gnutls_Write(cd, headers, sizeof(headers)) != sizeof(headers)) {
-			state->gnutls_internals.valid_connection = VALID_FALSE;
-			state->gnutls_internals.resumable = RESUME_FALSE;
-			gnutls_assert();
-			return GNUTLS_E_UNABLE_SEND_DATA;
-		}
+		/* cipher does not have headers 
+		 * and DOES have size for them
+		 */
+		memmove( cipher, headers, HEADER_SIZE);
+
+		cipher_size += HEADER_SIZE;
 		if (_gnutls_Write(cd, cipher, cipher_size) != cipher_size) {
 			state->gnutls_internals.valid_connection = VALID_FALSE;
 			state->gnutls_internals.resumable = RESUME_FALSE;
@@ -467,12 +496,9 @@ ssize_t gnutls_send_int(int cd, GNUTLS_STATE state, ContentType type, void *_dat
 		length = byteswap16(cipher_size);
 #endif
 		memmove( &headers[3], &length, sizeof(uint16));
-		if (_gnutls_Write(cd, headers, sizeof(headers)) != sizeof(headers)) {
-			state->gnutls_internals.valid_connection = VALID_FALSE;
-			state->gnutls_internals.resumable = RESUME_FALSE;
-			gnutls_assert();
-			return GNUTLS_E_UNABLE_SEND_DATA;
-		}
+		memmove( cipher, headers, HEADER_SIZE);
+
+		cipher_size += HEADER_SIZE;
 		if (_gnutls_Write(cd, cipher, cipher_size) != cipher_size) {
 			state->gnutls_internals.valid_connection = VALID_FALSE;
 			state->gnutls_internals.resumable = RESUME_FALSE;
@@ -505,11 +531,11 @@ ssize_t _gnutls_send_change_cipher_spec(int cd, GNUTLS_STATE state)
 	}
 
 	headers[0] = type;
-	headers[1] = state->connection_state.version.major;
-	headers[2] = state->connection_state.version.minor;
+	headers[1] = _gnutls_version_get_major(state->connection_state.version);
+	headers[2] = _gnutls_version_get_minor(state->connection_state.version);
 
 #ifdef HANDSHAKE_DEBUG
-	fprintf(stderr, "Send Change Cipher Spec\n");
+	fprintf(stderr, "ChangeCipherSpec was sent\n");
 #endif
 
 #ifdef WORDS_BIGENDIAN
@@ -555,8 +581,7 @@ char peekdata;
  * flags is the sockets flags to use. Currently only MSG_DONTWAIT is
  * supported.
  */
-#define HEADER_SIZE 5
-#define MAX_RECV_SIZE 18432 	/* 2^14+2048 */
+#define SSL2_HSIZE
 ssize_t gnutls_recv_int(int cd, GNUTLS_STATE state, ContentType type, char *data, size_t sizeofdata, int flags)
 {
 	uint8 *tmpdata;
@@ -567,7 +592,7 @@ ssize_t gnutls_recv_int(int cd, GNUTLS_STATE state, ContentType type, char *data
 	uint16 length;
 	uint8 *ciphertext;
 	int ret = 0;
-
+	int header_size = HEADER_SIZE;
 	/* If we have enough data in the cache do not bother receiving
 	 * a new packet. (in order to flush the cache)
 	 */
@@ -599,39 +624,57 @@ ssize_t gnutls_recv_int(int cd, GNUTLS_STATE state, ContentType type, char *data
 		return GNUTLS_E_UNEXPECTED_PACKET_LENGTH;
 	}
 
-	memcpy( &recv_type, &headers[0], 1);
-	memcpy( &version.major, &headers[1], 1);
-	memcpy( &version.minor, &headers[2], 1);
-	version.local = 0; /* TLS/SSL 3.0 */
+	/* Read the first two bytes to determine if this is a 
+	 * version 2 message 
+	 */
+	if ( headers[0] > 127 && type==GNUTLS_HANDSHAKE) { 
 
-	memcpy( &length, &headers[3], 2);
-#ifndef WORDS_BIGENDIAN
-	length = byteswap16(length);
+	/* if msb set and expecting handshake message
+	 * it should be SSL 2 hello 
+	 */
+		version = GNUTLS_SSL3; /* assume ssl 3.0 */
+		length = (((headers[0] & 0x7f) << 8)) | headers[1];
+		header_size = 2;
+		recv_type = GNUTLS_HANDSHAKE; /* only v2 client hello we accept */
+		state->gnutls_internals.v2_hello = length;
+#ifdef DEBUG
+		fprintf(stderr, "Record: V2 packet received. Length: %d\n", length);
 #endif
 
-	if (_gnutls_version_is_supported(state, version) == 0) {
+	} else {
+		/* version 3.x 
+		 */
+		memcpy( &recv_type, &headers[0], 1);
+		version = _gnutls_version_get( headers[1], headers[2]);
+
+		memcpy( &length, &headers[3], 2);
+#ifndef WORDS_BIGENDIAN
+		length = byteswap16(length);
+#endif
+	}
+	
+
+	if (type != GNUTLS_HANDSHAKE && gnutls_get_current_version(state) != version) {
 #ifdef DEBUG
-		fprintf(stderr, "INVALID VERSION PACKET: %d.%d\n", version.major, version.minor);
+		fprintf(stderr, "Record: INVALID VERSION PACKET: (%d) %d.%d\n", headers[0], headers[1], headers[2]);
 #endif
 		_gnutls_send_alert(cd, state, GNUTLS_FATAL, GNUTLS_PROTOCOL_VERSION);
 		state->gnutls_internals.resumable = RESUME_FALSE;
 		gnutls_assert();
 		return GNUTLS_E_UNSUPPORTED_VERSION_PACKET;
-	} else {
-		gnutls_set_current_version(state, version);
 	}
 
 
 #ifdef HARD_DEBUG
-	fprintf(stderr, "Expected Packet[%d] %s(%d) with length: %d\n",
+	fprintf(stderr, "Record: Expected Packet[%d] %s(%d) with length: %d\n",
 		(int) state->connection_state.read_sequence_number, _gnutls_packet2str(type), type, sizeofdata);
-	fprintf(stderr, "Received Packet[%d] %s(%d) with length: %d\n",
+	fprintf(stderr, "Record: Received Packet[%d] %s(%d) with length: %d\n",
 		(int) state->connection_state.read_sequence_number, _gnutls_packet2str(recv_type), recv_type, length);
 #endif
 
 	if (length > MAX_RECV_SIZE) {
 #ifdef DEBUG
-		fprintf(stderr, "FATAL ERROR: Received packet with length: %d\n", length);
+		fprintf(stderr, "Record: FATAL ERROR: Received packet with length: %d\n", length);
 #endif
 		_gnutls_send_alert(cd, state, GNUTLS_FATAL, GNUTLS_RECORD_OVERFLOW);
 		state->gnutls_internals.valid_connection = VALID_FALSE;
@@ -640,12 +683,12 @@ ssize_t gnutls_recv_int(int cd, GNUTLS_STATE state, ContentType type, char *data
 		return GNUTLS_E_UNEXPECTED_PACKET_LENGTH;
 	}
 
-	ciphertext = gnutls_malloc(length+HEADER_SIZE);
+	ciphertext = gnutls_malloc(length+header_size);
 
 /* check if we have that data into buffer. This seems to be
  * expensive - but this is the only way to handle Non Blocking IO.
  */
-	if ( _gnutls_Read(cd, ciphertext, HEADER_SIZE+length, MSG_PEEK|flags) != length+HEADER_SIZE) {
+	if ( _gnutls_Read(cd, ciphertext, header_size+length, MSG_PEEK|flags) != length+header_size) {
 		gnutls_free(ciphertext);
 		
 		if (errno==EAGAIN) return GNUTLS_E_AGAIN;
@@ -657,7 +700,7 @@ ssize_t gnutls_recv_int(int cd, GNUTLS_STATE state, ContentType type, char *data
 /* ok now we are sure that we can read all the data - so
  * move on !
  */
-	if (_gnutls_Read(cd, headers, HEADER_SIZE, 0)!=HEADER_SIZE) {  /* read and clear the headers - again! */
+	if (_gnutls_Read(cd, headers, header_size, 0)!=header_size) {  /* read and clear the headers - again! */
 		state->gnutls_internals.valid_connection = VALID_FALSE;
 		state->gnutls_internals.resumable = RESUME_FALSE;
 		gnutls_assert();
@@ -679,7 +722,7 @@ ssize_t gnutls_recv_int(int cd, GNUTLS_STATE state, ContentType type, char *data
 	 */
 	if (ret != length) {
 #ifdef DEBUG
-		fprintf(stderr, "Received packet with length: %d\nExpected %d\n", ret, length);
+		fprintf(stderr, "Record: Received packet with length: %d\nExpected %d\n", ret, length);
 #endif
 		gnutls_free(ciphertext);
 		state->gnutls_internals.valid_connection = VALID_FALSE;
@@ -690,7 +733,7 @@ ssize_t gnutls_recv_int(int cd, GNUTLS_STATE state, ContentType type, char *data
 	
 	if (type == GNUTLS_CHANGE_CIPHER_SPEC && recv_type == GNUTLS_CHANGE_CIPHER_SPEC) {
 #ifdef HARD_DEBUG
-		fprintf(stderr, "Received Change Cipher Spec Packet\n");
+		fprintf(stderr, "Record: ChangeCipherSpec Packet was received\n");
 #endif
 		if (length!=1) {
 			gnutls_assert();
@@ -728,7 +771,7 @@ ssize_t gnutls_recv_int(int cd, GNUTLS_STATE state, ContentType type, char *data
 		switch (recv_type) {
 		case GNUTLS_ALERT:
 #ifdef DEBUG
-			fprintf(stderr, "Alert[%d|%d] - %s - was received\n", tmpdata[0], tmpdata[1], _gnutls_alert2str((int)tmpdata[1]));
+			fprintf(stderr, "Record: Alert[%d|%d] - %s - was received\n", tmpdata[0], tmpdata[1], _gnutls_alert2str((int)tmpdata[1]));
 #endif
 			state->gnutls_internals.last_alert = tmpdata[1];
 
@@ -762,7 +805,7 @@ ssize_t gnutls_recv_int(int cd, GNUTLS_STATE state, ContentType type, char *data
 			break;
 		default:
 #ifdef DEBUG
-			fprintf(stderr, "Received Unknown packet %d expecting %d\n", recv_type, type);
+			fprintf(stderr, "Record: Received Unknown packet %d expecting %d\n", recv_type, type);
 #endif
 			gnutls_assert();
 			return GNUTLS_E_UNKNOWN_ERROR;
