@@ -22,9 +22,9 @@ enum ContentType { GNUTLS_APPLICATION_DATA=23 };
 typedef enum ContentType ContentType;
 #define GNUTLS_AES GNUTLS_RIJNDAEL
 
-enum BulkCipherAlgorithm { GNUTLS_NULL_CIPHER, GNUTLS_ARCFOUR=1, GNUTLS_3DES = 4, GNUTLS_RIJNDAEL, GNUTLS_TWOFISH };
+enum BulkCipherAlgorithm { GNUTLS_NULL_CIPHER, GNUTLS_ARCFOUR=1, GNUTLS_3DES = 4, GNUTLS_RIJNDAEL, GNUTLS_TWOFISH, GNUTLS_RIJNDAEL256 };
 typedef enum BulkCipherAlgorithm BulkCipherAlgorithm;
-enum KXAlgorithm { GNUTLS_KX_RSA, GNUTLS_KX_DHE_DSS, GNUTLS_KX_DHE_RSA, GNUTLS_KX_DH_DSS, GNUTLS_KX_DH_RSA, GNUTLS_KX_ANON_DH };
+enum KXAlgorithm { GNUTLS_KX_RSA, GNUTLS_KX_DHE_DSS, GNUTLS_KX_DHE_RSA, GNUTLS_KX_DH_DSS, GNUTLS_KX_DH_RSA, GNUTLS_KX_ANON_DH, GNUTLS_KX_SRP };
 typedef enum KXAlgorithm KXAlgorithm;
 enum MACAlgorithm { GNUTLS_NULL_MAC, GNUTLS_MAC_MD5, GNUTLS_MAC_SHA };
 typedef enum MACAlgorithm MACAlgorithm;
@@ -33,14 +33,9 @@ typedef enum CompressionMethod CompressionMethod;
 enum ConnectionEnd { GNUTLS_SERVER, GNUTLS_CLIENT };
 typedef enum ConnectionEnd ConnectionEnd;
 
-typedef struct {
-	unsigned char local;
-	unsigned char  major;
-	unsigned char  minor;
-} GNUTLS_Version;
-extern GNUTLS_Version GNUTLS_TLS1;
-extern GNUTLS_Version GNUTLS_SSL3;
-
+#define GNUTLS_Version int
+#define GNUTLS_TLS1 0
+#define GNUTLS_SSL3 1
 
 struct GNUTLS_STATE_INT;
 typedef struct GNUTLS_STATE_INT* GNUTLS_STATE;
@@ -79,11 +74,12 @@ char* gnutls_strerror(int error);
 
 /* functions to set priority of cipher suites */
 void gnutls_set_cipher_priority( GNUTLS_STATE state, int num, ...);
-void gnutls_set_kx_priority( GNUTLS_STATE state, int num, ...);
 void gnutls_set_mac_priority( GNUTLS_STATE state, int num, ...);
 void gnutls_set_compression_priority( GNUTLS_STATE state, int num, ...);
+void gnutls_set_kx_priority( GNUTLS_STATE state, int num, ...);
 
-/* set our version - local is 0x00 for TLS 1.0 and SSL3 */
+
+/* set our version - 0 for TLS 1.0 and 1 for SSL3 */
 void gnutls_set_current_version(GNUTLS_STATE state, GNUTLS_Version version); 
 
 /* get/set session */
@@ -97,6 +93,30 @@ int gnutls_set_cache_expiration( GNUTLS_STATE state, int seconds);
 int gnutls_set_db_name( GNUTLS_STATE state, char* filename);	
 int gnutls_clean_db( GNUTLS_STATE state);
 
+
+/* crypt functions */
+enum crypt_algo { SRPSHA1_CRYPT, BLOWFISH_CRYPT=2 };
+typedef enum crypt_algo crypt_algo;
+
+char * gnutls_crypt(const char* username, const char *passwd, crypt_algo algo, int salt);
+int gnutls_crypt_vrfy(const char* username, const char *passwd, char* salt);
+
+/* Functions for setting/clearing credentials */
+int gnutls_clear_creds( GNUTLS_STATE state);
+/* cred is a structure defined by the kx algorithm */
+int gnutls_set_kx_cred( GNUTLS_STATE, int kx, void* cred);
+
+/* Credential structures for SRP - used in gnutls_set_cred(); */
+typedef struct {
+	char* username;
+	char* password;
+} SRP_CLIENT_CREDENTIALS;
+
+typedef struct {
+	char* password_file;
+} SRP_SERVER_CREDENTIALS;
+
+
 /* these are deprecated must be replaced by gnutls_errors.h */
 #define	GNUTLS_E_MAC_FAILED  -1
 #define	GNUTLS_E_UNKNOWN_CIPHER -2
@@ -126,4 +146,11 @@ int gnutls_clean_db( GNUTLS_STATE state);
 #define GNUTLS_E_DECOMPRESSION_FAILED -26
 #define GNUTLS_E_COMPRESSION_FAILED -27
 #define GNUTLS_E_AGAIN -28
+#define GNUTLS_E_EXPIRED -29
+#define GNUTLS_E_DB_ERROR -30
+#define GNUTLS_E_PWD_ERROR -31
+#define GNUTLS_E_INSUFICIENT_CRED -32
+#define GNUTLS_E_HASH_FAILED -33
+#define GNUTLS_E_PARSING_ERROR -34
+#define	GNUTLS_E_MPI_PRINT_FAILED -35
 #define GNUTLS_E_UNIMPLEMENTED_FEATURE -50
