@@ -1,5 +1,30 @@
+/*
+ *      Copyright (C) 2000 Nikos Mavroyanopoulos
+ *
+ * This file is part of GNUTLS.
+ *
+ * GNUTLS is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * GNUTLS is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ */
+
+#include "gnutls_auth.h"
+
 /* functions for version */
 int _gnutls_version_is_supported(GNUTLS_STATE state, const GNUTLS_Version version);
+int _gnutls_version_get_major( GNUTLS_Version ver);
+int _gnutls_version_get_minor( GNUTLS_Version ver);
+GNUTLS_Version _gnutls_version_get( int major, int minor);
 
 /* functions for macs */
 int   _gnutls_mac_get_digest_size(MACAlgorithm algorithm);
@@ -34,10 +59,22 @@ char *_gnutls_cipher_get_name(BulkCipherAlgorithm algorithm);
 /* functions for key exchange */
 int _gnutls_kx_priority(GNUTLS_STATE state, KXAlgorithm algorithm);
 int _gnutls_kx_server_certificate(KXAlgorithm algorithm);
+
+/* key exchange */
 int _gnutls_kx_server_key_exchange(KXAlgorithm algorithm);
+int _gnutls_kx_server_key_exchange2(KXAlgorithm algorithm);
+int _gnutls_kx_client_key_exchange(KXAlgorithm algorithm);
+int _gnutls_kx_client_key_exchange0(KXAlgorithm algorithm);
+/* client certificate */
 int _gnutls_kx_client_certificate(KXAlgorithm algorithm);
 int _gnutls_kx_RSA_premaster(KXAlgorithm algorithm);
 int _gnutls_kx_DH_public_value(KXAlgorithm algorithm);
+
+/* cert vrfy */
+int _gnutls_kx_client_cert_vrfy(KXAlgorithm algorithm);
+int _gnutls_kx_server_cert_vrfy(KXAlgorithm algorithm);
+
+MOD_AUTH_STRUCT * _gnutls_kx_auth_struct(KXAlgorithm algorithm);
 char *_gnutls_kx_get_name(KXAlgorithm algorithm);
 int _gnutls_kx_is_ok(KXAlgorithm algorithm);
 int _gnutls_kx_count();
