@@ -28,6 +28,7 @@
 #include "gnutls_cipher_int.h"
 #include "gnutls_plaintext.h"
 #include "debug.h"
+#include "gnutls_random.h"
 
 int _gnutls_encrypt(GNUTLS_STATE state, char *data, size_t data_size,
 		    uint8 ** ciphertext, ContentType type)
@@ -35,7 +36,7 @@ int _gnutls_encrypt(GNUTLS_STATE state, char *data, size_t data_size,
 	GNUTLSPlaintext *gtxt;
 	GNUTLSCompressed *gcomp;
 	GNUTLSCiphertext *gcipher;
-	int total_length = 0, err;
+	int total_length, err;
 
 	if (data_size == 0)
 		return 0;
@@ -63,18 +64,19 @@ int _gnutls_encrypt(GNUTLS_STATE state, char *data, size_t data_size,
 
 	_gnutls_freeTLSCompressed(gcomp);
 
-	*ciphertext = gnutls_malloc(gcipher->length);
+	total_length = HEADER_SIZE + gcipher->length;
+	*ciphertext = gnutls_malloc(total_length);
 	if (*ciphertext == NULL) {
 		gnutls_assert();
 		return GNUTLS_E_MEMORY_ERROR;
 	}
-	memmove((*ciphertext), gcipher->fragment, gcipher->length);
-
-
-	total_length += gcipher->length;
+	memmove( &(*ciphertext)[HEADER_SIZE], gcipher->fragment, gcipher->length);
 	_gnutls_freeTLSCiphertext(gcipher);
 
-	return total_length;
+	return total_length - HEADER_SIZE;
+	/* notice that the headers are not present in the retun value,
+	 * but there is space for them
+	 */
 }
 
 int _gnutls_decrypt(GNUTLS_STATE state, char *ciphertext,
@@ -91,8 +93,8 @@ int _gnutls_decrypt(GNUTLS_STATE state, char *ciphertext,
 
 	gcipher.type = type;
 	gcipher.length = ciphertext_size;
-	gcipher.version.major = state->connection_state.version.major;
-	gcipher.version.minor = state->connection_state.version.minor;
+	gcipher.version.major = _gnutls_version_get_major(state->connection_state.version);
+	gcipher.version.minor = _gnutls_version_get_minor(state->connection_state.version);
 	gcipher.fragment = gnutls_malloc(ciphertext_size);
 	memmove(gcipher.fragment, ciphertext, ciphertext_size);
 
@@ -490,7 +492,7 @@ int _gnutls_TLSCompressed2TLSCiphertext(GNUTLS_STATE state,
 
 		break;
 	case CIPHER_BLOCK:
-		rand = gcry_random_bytes(1, GCRY_WEAK_RANDOM);
+		rand = _gnutls_get_random(1, GNUTLS_WEAK_RANDOM);
 
 		/* make rand a multiple of blocksize */
 		if (_gnutls_version_ssl3(state->connection_state.version) == 0) {
@@ -522,7 +524,7 @@ int _gnutls_TLSCompressed2TLSCiphertext(GNUTLS_STATE state,
 		ciphertext->version.major = compressed->version.major;
 		ciphertext->version.minor = compressed->version.minor;
 
-		gcry_free(rand);
+		_gnutls_free_rand(rand);
 		break;
 	default:
 		gnutls_free(*cipher);
