@@ -33,7 +33,6 @@ int gnutls_set_cache_expiration( GNUTLS_STATE state, int seconds) {
 
 int gnutls_set_db_name( GNUTLS_STATE state, char* filename) {
 #ifdef HAVE_LIBGDBM
-GDBM_FILE dbf;
 
 	gnutls_free(state->gnutls_internals.db_name);
 	state->gnutls_internals.db_name = strdup(filename);
@@ -112,3 +111,23 @@ int ret;
 #endif
 }
 
+int _gnutls_db_remove_session( GNUTLS_STATE state, uint8* session_id, int session_id_size)
+{
+#ifdef HAVE_LIBGDBM
+GDBM_FILE dbf;
+datum key = { session_id, session_id_size};
+int ret;
+
+	if (state->gnutls_internals.db_name==NULL) return GNUTLS_E_DB_ERROR;
+
+	dbf = gdbm_open(state->gnutls_internals.db_name, 0, GDBM_READER|GDBM_FAST, 0600, NULL);
+	if (dbf==NULL) return GNUTLS_E_AGAIN;
+	ret = gdbm_delete( dbf, key);
+	gdbm_close(dbf);
+
+	return (ret==0 ? ret : GNUTLS_E_DB_ERROR);
+#else
+	return GNUTLS_E_UNIMPLEMENTED_FEATURE;
+#endif
+}
+
