@@ -1,11 +1,32 @@
+/*
+ *      Copyright (C) 2000 Nikos Mavroyanopoulos
+ *
+ * This file is part of GNUTLS.
+ *
+ * GNUTLS is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * GNUTLS is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ */
+
 #ifndef GNUTLS_INT_H
 
 #define GNUTLS_INT_H
 
 /*
-#define HANDSHAKE_DEBUG
 #define READ_DEBUG
 #define WRITE_DEBUG
+#define BUFFERS_DEBUG
+#define HANDSHAKE_DEBUG
 #define HARD_DEBUG
 #define DEBUG
 */
@@ -14,8 +35,10 @@
 #define MAX24 16777215
 #define MAX16 65535
 
-/* for big numbers support */ /* FIXME */
-#include <gcrypt.h>
+/* the maximum size of encrypted packets */
+#define MAX_ENC_LEN 16384
+#define HEADER_SIZE 5
+#define MAX_RECV_SIZE 18432+HEADER_SIZE 	/* 2^14+2048+HEADER_SIZE */
 
 #ifdef USE_DMALLOC
 #include <dmalloc.h>
@@ -50,7 +73,7 @@ typedef struct {
 
 typedef unsigned char opaque;
 
-
+enum crypt_algo { SRPSHA1_CRYPT, BLOWFISH_CRYPT=2 };
 enum ChangeCipherSpecType { GNUTLS_TYPE_CHANGE_CIPHER_SPEC=1 };
 enum AlertLevel { GNUTLS_WARNING=1, GNUTLS_FATAL };
 enum AlertDescription { GNUTLS_CLOSE_NOTIFY, GNUTLS_UNEXPECTED_MESSAGE=10, GNUTLS_BAD_RECORD_MAC=20,
@@ -66,6 +89,7 @@ enum AlertDescription { GNUTLS_CLOSE_NOTIFY, GNUTLS_UNEXPECTED_MESSAGE=10, GNUTL
 typedef enum AlertDescription AlertDescription;
 typedef enum AlertLevel AlertLevel;
 typedef enum ChangeCipherSpecType ChangeCipherSpecType;
+typedef enum crypt_algo crypt_algo;
 
 enum HandshakeType { GNUTLS_HELLO_REQUEST, GNUTLS_CLIENT_HELLO, GNUTLS_SERVER_HELLO,
 		     GNUTLS_CERTIFICATE=11, GNUTLS_SERVER_KEY_EXCHANGE,
@@ -88,8 +112,9 @@ typedef struct {
 
 /* STATE */
 enum ConnectionEnd { GNUTLS_SERVER, GNUTLS_CLIENT };
-enum BulkCipherAlgorithm { GNUTLS_NULL_CIPHER, GNUTLS_ARCFOUR=1, GNUTLS_3DES = 4, GNUTLS_RIJNDAEL, GNUTLS_TWOFISH };
-enum KXAlgorithm { GNUTLS_KX_RSA, GNUTLS_KX_DHE_DSS, GNUTLS_KX_DHE_RSA, GNUTLS_KX_DH_DSS, GNUTLS_KX_DH_RSA, GNUTLS_KX_ANON_DH };
+enum BulkCipherAlgorithm { GNUTLS_NULL_CIPHER, GNUTLS_ARCFOUR=1, GNUTLS_3DES = 4, GNUTLS_RIJNDAEL, GNUTLS_TWOFISH, GNUTLS_RIJNDAEL256 };
+enum Extensions { GNUTLS_EXTENSION_SRP=7 };
+enum KXAlgorithm { GNUTLS_KX_RSA, GNUTLS_KX_DHE_DSS, GNUTLS_KX_DHE_RSA, GNUTLS_KX_DH_DSS, GNUTLS_KX_DH_RSA, GNUTLS_KX_ANON_DH, GNUTLS_KX_SRP };
 enum KeyExchangeAlgorithm { GNUTLS_RSA, GNUTLS_DIFFIE_HELLMAN };
 enum CipherType { CIPHER_STREAM, CIPHER_BLOCK };
 enum MACAlgorithm { GNUTLS_NULL_MAC, GNUTLS_MAC_MD5, GNUTLS_MAC_SHA };
@@ -107,9 +132,48 @@ typedef enum BulkCipherAlgorithm BulkCipherAlgorithm;
 typedef enum CipherType CipherType;
 typedef enum MACAlgorithm MACAlgorithm;
 typedef enum CompressionMethod CompressionMethod;
+typedef enum Extensions Extensions;
+
+/* STATE (stop) */
+
+typedef struct {
+	KXAlgorithm algorithm;
+	void* credentials;
+	void* next;
+} AUTH_CRED;
+
+typedef struct {
+	/* For DH KX */
+	MPI				KEY;
+	MPI				client_Y;
+	MPI				client_g;
+	MPI				client_p;
+	MPI				dh_secret;
+	/* for SRP */
+	MPI				A;
+	MPI				B;
+	MPI				u;
+	MPI				b;
+	MPI				a;
+	MPI				x;
+	
+	/* this is only used in server side - 
+	 * to hold the requested user - currently only in srp 
+	 */
+	char*				username;
+	uint8				crypt_algo;
+	
+	AUTH_CRED*			cred; /* used in srp, etc */
+} GNUTLS_KEY_A;
+typedef GNUTLS_KEY_A* GNUTLS_KEY;
+
+
+/* STATE (cont) */
 
 #include <gnutls_hash_int.h>
 #include <gnutls_cipher_int.h>
+#include <gnutls_auth.h>
+
 
 typedef struct {
 	ConnectionEnd entity;
@@ -139,17 +203,12 @@ typedef struct {
 } CipherSpecs;
 
 
-typedef struct {
-	uint8 local;
-	uint8 major;
-	uint8 minor;
-} GNUTLS_Version;
-
-extern GNUTLS_Version GNUTLS_TLS1;
-extern GNUTLS_Version GNUTLS_SSL3;
+#define GNUTLS_Version int
+#define GNUTLS_TLS1 0
+#define GNUTLS_SSL3 1
 
 typedef struct {
-	GNUTLS_Version version;
+	GNUTLS_Version	version;
 	opaque* 	read_compression_state;
 	opaque* 	write_compression_state;
 	GNUTLS_CIPHER_HANDLE write_cipher_state;
@@ -157,8 +216,8 @@ typedef struct {
 	opaque* 	read_mac_secret;
 	opaque* 	write_mac_secret;
 	uint8   	mac_secret_size;
-	uint64	read_sequence_number;
-	uint64	write_sequence_number;
+	uint64		read_sequence_number;
+	uint64		write_sequence_number;
 } ConnectionState;
 
 typedef struct {
@@ -197,12 +256,7 @@ typedef struct {
 	/* resumed session */
 	ResumableSession	resumed; /* TRUE or FALSE - if we are resuming a session */
 	SecurityParameters  resumed_security_parameters;
-	/* For DH KX */
-	MPI				KEY;
-	MPI				client_Y;
-	MPI				client_g;
-	MPI				client_p;
-	MPI				dh_secret;
+
 	int				certificate_requested; /* non zero if client certificate was requested */
 	int				certificate_verify_needed; /* non zero if we should expect for certificate verify */
 	/* sockets internals */
@@ -210,6 +264,8 @@ typedef struct {
 	/* gdbm */
 	char*				db_name;
 	int				expire_time;
+	MOD_AUTH_STRUCT*		auth_struct; /* used in handshake packets and KX algorithms */
+	int				v2_hello; /* set 0 normally - 1 if v2 hello was received - server side only */
 } GNUTLS_INTERNALS;
 
 typedef struct {
@@ -217,6 +273,7 @@ typedef struct {
 	CipherSpecs cipher_specs;
 	ConnectionState connection_state;
 	GNUTLS_INTERNALS gnutls_internals;
+	GNUTLS_KEY gnutls_key;
 } GNUTLS_STATE_INT;
 
 typedef GNUTLS_STATE_INT *GNUTLS_STATE;
@@ -318,4 +375,6 @@ int _gnutls_send_change_cipher_spec(int cd, GNUTLS_STATE state);
 int _gnutls_version_cmp(GNUTLS_Version ver1, GNUTLS_Version ver2);
 #define _gnutls_version_ssl3(x) _gnutls_version_cmp(x, GNUTLS_SSL3)
 
+#define gcry_mpi_alloc_like(x) gcry_mpi_new(gcry_mpi_get_nbits(x)) 
+
 #endif /* GNUTLS_INT_H */
