@@ -43,8 +43,13 @@ int main()
     char topbuf[512];
     GNUTLS_STATE state;
     char buffer[MAX_BUF+1];
-
-
+    int optval = 1;
+    SRP_SERVER_CREDENTIALS cred;
+    
+    /* this is a password file (created with the included crypt utility) 
+     */
+    cred.password_file="/tmp/pwd";
+    
     listen_sd = socket(AF_INET, SOCK_STREAM, 0);
     ERR(listen_sd, "socket");
 
@@ -53,6 +58,7 @@ int main()
     sa_serv.sin_addr.s_addr = INADDR_ANY;
     sa_serv.sin_port = htons(PORT);	/* Server Port number */
 
+    setsockopt( listen_sd, SOL_SOCKET, SO_REUSEADDR, &optval, sizeof(int));
     err = bind(listen_sd, (SA *) & sa_serv, sizeof(sa_serv));
     ERR(err, "bind");
     err = listen(listen_sd, 1024);
@@ -66,7 +72,11 @@ int main()
 	gnutls_set_db_name(state, "/tmp/gdb");
 	gnutls_set_cipher_priority( state, 4, GNUTLS_TWOFISH, GNUTLS_RIJNDAEL, GNUTLS_3DES, GNUTLS_ARCFOUR);
 	gnutls_set_compression_priority( state, 2, GNUTLS_ZLIB, GNUTLS_NULL_COMPRESSION);
-	gnutls_set_kx_priority( state, 1, GNUTLS_KX_ANON_DH);
+	gnutls_set_kx_priority( state, 2, GNUTLS_KX_SRP, GNUTLS_KX_ANON_DH);
+	
+	gnutls_set_kx_cred( state, GNUTLS_KX_ANON_DH, NULL);
+	gnutls_set_kx_cred( state, GNUTLS_KX_SRP, &cred);
+	
 	gnutls_set_mac_priority( state, 2, GNUTLS_MAC_SHA, GNUTLS_MAC_MD5);
 	sd = accept(listen_sd, (SA *) & sa_cli, &client_len);
 
